/**
* Copyright (c) 2020
* HLRS (Supercomputer Center Stuttgart)
*
* @summary Code for interaction of the Web Interface with the RESTful interface of the WMS server
* @author Jose Miguel Montañana <montanana@hlrs.de>
* @version 1.0
*
* Last modified : 2020-03-29
*/

function getType(p) {
	if (Array.isArray(p)) return 'array';
	else if (typeof p == 'string') return 'string';
	else if (p != null && typeof p == 'object') return 'object';
	else return 'other';
}

// In case it is not defined, we define the function here.
if (!String.prototype.endsWith) {
	String.prototype.endsWith = function(searchString, position) {
		var subjectString = this.toString();
		if (typeof (position) !== 'number' || !isFinite(position) || Math.floor(position) !== position || position> subjectString.length) {
			position = subjectString.length;
		}
		position -= searchString.length;
		var lastIndex = subjectString.indexOf(searchString, position);
		return lastIndex !== -1 && lastIndex === position;
	};
}





function switchnightmode(){
	var body = document.getElementById("body");
	var table_results = document.getElementById("table_results");
	var currentClass = body.className;
	body.className = currentClass == "dark-mode" ? "light-mode" : "dark-mode";
		if(table_results!=null)
	table_results.className = body.className;
	localStorage.setItem('currentmode', body.className);
	var c = document.getElementById("foot_wms").querySelectorAll("a");
	for (i in c) {
		c[i].className = body.className;
	}
	//update it at last
	document.getElementById("foot_wms").className = body.className;
}


function load_footer(){
	var foot_wms = document.getElementById("foot_wms");
	if(foot_wms){
	var menuhtml ="";

	menuhtml+="<div class=\"greyfont\">WMS project: 2020<br/>";
	menuhtml+="	Licensed under the Apache License, Version 2.0<br/>";
	menuhtml+="	You may obtain a copy of the License at:<br/>";
	menuhtml+="	<a href=\"http://www.apache.org/licenses/LICENSE-2.0\">";
	menuhtml+="	http://www.apache.org/licenses/LICENSE-2.0</a>";
	menuhtml+="	</div>";
	foot_wms.innerHTML = menuhtml;
	}
}



function pad(num, size) {
	var s = num+"";
	while (s.length < size) s = "0" + s;
	return s;
}


function calculate_date(input) {
	var current = input;
	current = Math.floor(current /1000000);
	var tempstr="";
	var months=[31, 28, 31, 30, 31, 30, 31, 31, 30 ,31 ,30 ,31 ];
	var exampledate_msec =Math.floor(current % 1000);
	current= Math.floor(current /1000);
	var exampledate_sec = Math.floor(current %60);
	current =Math.floor(current / 60);
	var exampledate_min = Math.floor(current % 60);
	current = Math.floor(current /60);
	var exampledate_hour = Math.floor(current %24);
	current = Math.floor(current /24);
	var exampledate_year=0;
	if(current > 365+365+366){
		exampledate_year=3;
		current -= (365+365+366);
	}
	if(current > 1461){
		exampledate_year+=(4*Math.floor(current / 1461));
		current = Math.floor(current % 1461);
	}
	exampledate_year+=Math.floor(current / 365);
	current = Math.floor(current % 365);
	var leap_year;
	leap_year= (exampledate_year %4==2)? 1 : 0;
	if (leap_year==1) months[1]=29;
	var exampledate_month=0;
	while(current> months[exampledate_month]){
		current -= months[exampledate_month];
		exampledate_month++;
	}
	exampledate_year= 1970+ Math.floor(exampledate_year);
	var exampledate_day= 1+ current;
	exampledate_month=1+exampledate_month;
	tempstr=exampledate_year+"-"+pad(exampledate_month,2)+"-"+pad(exampledate_day,2)+"T"
	+pad(exampledate_hour,2)+":"+pad(exampledate_min,2)+":"+pad(exampledate_sec,2)+"."+pad(exampledate_msec,2);
	return tempstr;
}



function str2bytes (str) {
	var bytes = new Uint8Array(str.length);
	for (var i=0; i<str.length; i++) {
		bytes[i] = str.charCodeAt(i);
	}
	return bytes;
}

function ab2str(buf) {
	return String.fromCharCode.apply(null, new Uint16Array(buf));
}

function str2ab(str) {
	var buf = new ArrayBuffer(str.length*2); // 2 bytes for each char
	var bufView = new Uint16Array(buf);
	for (var i=0, strLen=str.length; i < strLen; i++) {
		bufView[i] = str.charCodeAt(i);
	}
	return buf;
}


function add_tabs(level,count,label){
	var html="";
	html += "<br>";
// 	html += "["+level+" , "+ count+"]"+label;
	for (i = 0; i < level; i++) {
		html += '&emsp;';
	}
	return html;
}

function jsontohtml(myjson,count,first,level,lastwascoma,filtered_fields){
	var html ="";
	var i;
	var countseries=0;
	var countb=1;

	if(level==1)
// 	html += add_tabs(0,countseries,"----");
	if(first==true){
		html += add_tabs(level,1,"aaaa");
		html += '{';
	}

	myjson.forEach(function(val) {
		countb=1;
		if (count != 1 && lastwascoma==false) {
			if(countseries!=0) {
				html += add_tabs(level,countb,"bbbb");
				html += "},";
// 				html += add_tabs(0,countseries,"++++");
				html += add_tabs(level,countb,"bbbb");
				html += "{";
			}
		};
		count++;
		lastwascoma=true;
		var keys = Object.keys(val);
		keys.forEach(function(key) {
			if (getType(val[key]) == "string" || getType(val[key]) == "other"){//other can be a numeric value
				var tobefiltered=false;
				for (i=0;i< filtered_fields.length;i++){
					if (key.endsWith(filtered_fields[i], key.length)== true) {
						tobefiltered=true;
					}
				}
				if (tobefiltered== false) {//it is stored the length of the strings, not need to show
					if (countb != 1 && lastwascoma==false) {
						html += ',';
					}
					if (countb != 1)
						html += add_tabs(level,countb,"cccc");
					html += "<strong>\"" + key + "\"</strong>: \"" + val[key] +"\"";
			        countb++;
					lastwascoma=false;
				}
			}else if (getType(val[key]) == "array" ) {
					if (countb != 1) html += ',';
					html += add_tabs(level,countb,"dddd");
					html += "<strong>\"" + key + "\"</strong>:";lastwascoma=false;
					countb++;
					html += jsontohtml( ([ val[key] ]), 1, true, level+1 ,lastwascoma,filtered_fields) +"\n";
			}else if (getType(val[key]) == "object" ) {
// 				html += "<tr><td><strong> &emsp;" + key + "</strong>: \"" + JSON.stringify(val[key]) +"\"</td>\n";//this shows a key counter

				if (countb != 1){
					html += add_tabs(level,countb,"tttt");
					html += ',';
				}
				html += "<strong>\"" + key + "\"</strong>:";lastwascoma=false;
				html += add_tabs(level+1,countb,"pppp");
				html += "{";
				countb++;
				html += jsontohtml( ([ val[key] ]), 1, false, level+1,lastwascoma,filtered_fields) +"\n";
				html += add_tabs(level+1,countb,"ffff");
				html += '}';
			};
		});
		countseries++;
	});
	if(first==true){
		html += add_tabs(level,countb,"zzzz");
		html += '}';
	}
	return html;
}


function jsontotable_repo_logs_brief(typeserver, myjson,count,first,level,lastwascoma,mtitle,filtered_fields){
	var html ="";
	var i;
	var mainc=mtitle;
	if(first==true){
		html += "<div><table style='border:1px solid black' id=\"table_results\">\n";// style='width:100%'>";
		var typevalue=0;
		var myfunction="";
		if(typeserver==1){//repository
			typevalue=0;
			myfunction="list_repo_logs";
		}else if(typeserver==2){//app-manager
			typevalue=10;
			myfunction="list_app_logs";
		}else if(typeserver==3){// exec-manager's
			typevalue=20;
			myfunction="list_exec_logs";
		}else if(typeserver==4){//monitoring server
			typevalue=30;
			myfunction="list_mf_logs";
		}else if(typeserver==5){// resource-manager's logs
			typevalue=40;
			myfunction="list_resource_logs";
		}
		var myvalue=201+typevalue;
		html += "<td>#</td><td align=\"center\"><a onclick=\"return "+myfunction+"("+myvalue+",document.getElementById('username').value)\" class=\"under-logs\">_id</a></td>\n";
		var myvalue=202+typevalue;
		html += "<td align=\"center\">&nbsp;<a onclick=\"return "+myfunction+"("+myvalue+",document.getElementById('username').value)\" class=\"under-logs\">Code</a>&nbsp;</td>\n";
		var myvalue=203+typevalue;
		html += "<td align=\"center\">&nbsp;<a onclick=\"return "+myfunction+"("+myvalue+",document.getElementById('username').value)\" class=\"under-logs\">User</a>&nbsp;</td>\n";
		var myvalue=204+typevalue;
		html += "<td align=\"center\">&nbsp;<a onclick=\"return "+myfunction+"("+myvalue+",document.getElementById('username').value)\" class=\"under-logs\">Ip</a>&nbsp;</td>\n";
		var myvalue=205+typevalue;
		html += "<td align=\"center\">&nbsp;<a onclick=\"return "+myfunction+"("+myvalue+",document.getElementById('username').value)\" class=\"under-logs\">Message</a>&nbsp;</td>\n";
		var myvalue=200+typevalue;
		html += "<td align=\"center\">&nbsp;<a onclick=\"return "+myfunction+"("+myvalue+",document.getElementById('username').value)\" class=\"under-logs\">Date</a>&nbsp;</td>\n";
		count++;
	}
	first=false;
	var countseries=0;
	myjson.forEach(function(val) {
		lastwascoma=true;
		var keys = Object.keys(val);
		keys.forEach(function(key) {
			if (getType(val[key]) == "string" || getType(val[key]) == "other" ){
				var tobefiltered=false;
				for (i=0;i< filtered_fields.length;i++){
					if (key.endsWith(filtered_fields[i], key.length)== true) {
						tobefiltered=true;
					}
				}
				if (tobefiltered== false) {//it is stored the length of the strings, not need to show
					if(mtitle==true){
						if(count>0){
							html += "</tr>\n<tr>";
						}
						html += "<td>"+count+"</td><th>" + val['_id'] +"</th>\n";
						//source
						if(val['_source'] !=undefined){
							if(val['_source']['code']==undefined){
								html += "<td>";
							}else if((val['_source']['code']>="200")&&(val['_source']['code']<"300")){//green 2xx-correct,3xx-redirections
								html += "<td bgcolor=\"#00ff00\"> <font color=\"black\">" + val['_source']['code'] +"</font>";
							}else if((val['_source']['code']>="400") && (val['_source']['code']<"600")) {//red 4xx-client-error 5xx-server-error
								html += "<td bgcolor=\"#ff3e29\"> <font color=\"black\">" + val['_source']['code'] +"</font>";
							}else if((val['_source']['code']>="100")&&(val['_source']['code']<"200")){ //yellow-information
								html += "<td bgcolor=\"#f3ff3a\"> <font color=\"black\">" + val['_source']['code'] +"</font>";

							}
							html += "</td>\n<td>"; html += (val['_source']['user']==undefined)? "" : val['_source']['user'];
							html += "</td>\n<td>"; html += (val['_source']['ip']==undefined)? "" : val['_source']['ip'];
							html += "</td>\n<td>"; html += (val['_source']['message']==undefined)? "" : val['_source']['message'];
							html += "</td>\n<td>"; html += (val['_source']['date']==undefined)? "" : val['_source']['date'];
							html += "</td>\n";
						}else{
							html += "<td></td>\n";
							html += "<td></td>\n";
							html += "<td></td>\n";
							html += "<td></td>\n";
							html += "<td></td>\n";
						}
						mtitle=false;
						lastwascoma=false;
					}
				}
			}else if (getType(val[key]) == "array" || getType(val[key]) == "object" ) {
					html += jsontotable_repo_logs_brief(typeserver, ([ val[key] ]), count, first, level+1 ,lastwascoma,mtitle,filtered_fields);
					count++;
			};
		});
		countseries++;
	});
	return html;
}

function upload_with_token( UploadJSON, url ) {
	var demoreplaceb = document.getElementById("demoreplaceb");
	var debug_wms = document.getElementById("debug_wms");
		var xhr = new XMLHttpRequest();
		var formData = new FormData();
		xhr.open('POST', url, true);
		xhr.addEventListener('load', function() {
			var responseObject = (xhr.responseText);
			if(demoreplaceb) demoreplaceb.innerHTML = "<pre>"+ responseObject + " status: " +xhr.status+ "</pre>";
			if(debug_wms) debug_wms.style.display = "block";
		});
		formData.append("UploadJSON", UploadJSON.files[0]);
		xhr.send(formData);//may fault code appear here
	return false;
}

function list_results_with_token( mytype ,url,fields_toshow, filtered_fields) {
	var demoreplaceb = document.getElementById("demoreplaceb");
	var debug_wms = document.getElementById("debug_wms");
		list_results(mytype,url,fields_toshow,filtered_fields);
	return false;
}

function jsontotable(myjson,count,first,level,lastwascoma,mtitle,filtered_fields){
	var html ="";
	var i;
	var mainc=mtitle;
	if(mtitle==true){
		html += "<div><table style='border:1px solid black'>\n";// style='width:100%'>";
	}
	var countseries=0;
	myjson.forEach(function(val) {
		lastwascoma=true;
		var keys = Object.keys(val);
		keys.forEach(function(key) {
			if (getType(val[key]) == "string" || getType(val[key]) == "other" ){
				var tobefiltered=false;
				for (i=0;i< filtered_fields.length;i++){
					if (key.endsWith(filtered_fields[i], key.length)== true) {
						tobefiltered=true;
					}
				}
				if (tobefiltered== false) {//it is stored the length of the strings, not need to show
					if(mtitle==true){
						if(count>1){
							html += "</table></div></td><br>\n";
							html += "<div><table style='border:1px solid black'>\n";// style='width:100%'>";
						}
						html += "<tr><th><strong>\""+ key +"\"</strong>: \"" + val[key] +"\"</th></tr>\n";
					}else{
						html += "<tr><td><strong>\"" + key +"\"</strong>: \"" + val[key] +"\"</td></tr>\n";
					}
					mtitle=false;
					count++;
					lastwascoma=false;
				}
			}else if (getType(val[key]) == "array" || getType(val[key]) == "object" ) {
					if(mtitle==true){
						if(count>1){
							html += "</table></div></td><br>\n";
							html += "<div><table style='border:1px solid black'>\n";// style='width:100%'>";
						}
						html += "<tr><th><strong>\"" + key + "\"</strong>:</th>\n";
					}else{
						html += "<tr><td><strong>\"" + key + "\"</strong>:</td>\n";
					}
					mtitle=false;
					count++;
					lastwascoma=false;
					html += "<td><div><table style='width:100%; border:0px solid black'>\n";// style='width:100%'>";
					html += jsontotable( ([ val[key] ]), count, true, level+1 ,lastwascoma,mtitle,filtered_fields);
					html += "</table></div></td>\n";
			};
		});
		mtitle=true;
		countseries++;
	});
	if(mainc==true)
		html += "</table></div>\n";
	return html;
}//jsontotable


function list_results(mytype, url,fields_toshow,filtered_fields){
	var demoreplaceb = document.getElementById("demoreplaceb");
	var debug_wms = document.getElementById("debug_wms");
	var xhr = new XMLHttpRequest();
	xhr.open("GET", url, true);
	xhr.addEventListener('load', function() {
		var html = "";// to store the conversion of json to html format
		if (xhr.readyState === 4 && xhr.status == 200) {
			var responseObject = [ xhr.responseText ];
			var myjson = JSON.parse(responseObject || '{}');
			if(myjson.hits!=undefined) {
				myjson = myjson.hits;
			}else{
				myjson = [ myjson ];
			}
			if(myjson!=undefined) {
				if (mytype== 1) {
					html += jsontotable(myjson,1,true,1,false,true,filtered_fields);
				}else if ((mytype == 5) || ((mytype > 800) && (mytype < 812 ))){ //8XX for sorting
					html += jsontotable_app_brief(myjson,1,true,1,false,true,filtered_fields);
				}else if ((mytype == 15) || ((mytype > 800) && (mytype < 812 ))){ //8XX for sorting
					html += jsontotable_app_depl_brief(myjson,1,true,1,false,true,filtered_fields);
				}else if ((mytype == 25) || ((mytype > 800) && (mytype < 812 ))){ //8XX for sorting
					html += jsontotable_em_brief(myjson,1,true,1,false,true,filtered_fields);
				}else if ((mytype == 35) || ((mytype > 800) && (mytype < 812 ))){ //8XX for sorting
					html += jsontotable_rm_brief(myjson,1,true,1,false,true,filtered_fields);
				}else if (mytype == 2){
					html += "<font face='Courier New'>";
					html += jsontohtml(myjson,1,true,1,false,filtered_fields);
					html += "</font>";
				}else{ //3
					html += "<p align=\"justify\">"+ JSON.stringify(myjson)+ "</p>";
				}
			}
			if (demoreplaceb) demoreplaceb.innerHTML = html; //+responseObject + " status: " +xhr.status;
			if (debug_wms) debug_wms.style.display = "block";
		}
	});
	xhr.send(null);
	return false;
}



function download_file(content, fileName, contentType) {
	var file = new Blob([content], {type: contentType});
		var a = document.createElement("a");
		document.body.appendChild(a);
		var url = URL.createObjectURL(file);
		a.style = "display:none";
		a.href = url;
		a.download = fileName;
		a.click();
		a.remove();
}


/**
* @return a file with the server response if a outputfilename is provided
* */
function submitform(url, operation, outputfile) {
	var demoreplaceb = document.getElementById("demoreplaceb");
	var debug_wms = document.getElementById("debug_wms");
	var wms_operation = document.getElementById("wms_operation");
		var xhr = new XMLHttpRequest();
		xhr.open(operation, url, true);
		xhr.setRequestHeader('X-Requested-With','XMLHttpRequest');
		xhr.addEventListener('load', function() {
			if (xhr.readyState == 4) {
				var r = "";
				if (xhr.status == 200 || xhr.status == 0) {
					if(outputfile.length>0)
						download_file(xhr.responseText, outputfile , 'text/plain');
					r = "Server response:<br/><br/>"+xhr.responseText+"<br/>";
				}else{
					r = "Error " + xhr.status + " occurred requesting for Metatada.<br/>";
				}
				if(demoreplaceb) document.getElementById("demoreplaceb").innerHTML = "<pre>"+r+"</pre>";
				if(debug_wms) document.getElementById("debug_wms").style.display = "block";
				if(wms_operation) document.getElementById("wms_operation").style.display = "none";
			}
		});
		xhr.send(null);//may fault code appear here
	return false;
}
