//author: J.M.Montanana
//Copyright HRLS 2017
//This functions are for printing in the screen the processing values in stats.c
//its purpose are only for disertation and debugging
//based on the use of ps and /proc/stat , /proc/diskstats, /proc/uptime, /proc/meminfo, /proc/[pid]/io, nproc

#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <stdbool.h>
#include <sys/ioctl.h>//to get the screen resolution
#include <unistd.h> //for sleep and usleep
#include "mystrings.h"
#include "stats_data.h"

#include <time.h>
void print_memorysize(long long int value){
	if(value> 1024*1024*1024){
		printf("%5lliGB", value/(1024*1024*1024));
	}else if(value> 1024*1024){
		printf("%5lliMB", value/(1024*1024));
	}else if(value> 1024){
		printf("%5lliKB", value/1024);
	}else {
		printf("%5lli B", value);
	}
}

//cleans the screen and positionates the cursor on the position 6,3
//Note: it not works on some terminals
void cls(void) {
	printf("\33[2J");printf("\033[6;3H");
}

void emptyline(const unsigned int totallines,const unsigned int lengthline){
	for(int j=0;j<totallines;j++){
		for(int i=0;i<lengthline;i++)
			printf(" ");
		printf("\n");
	}
}

void print_disk_stats( struct disk_data *newdata, struct disk_data *olddata, int totaldevices, const int col, long int itv ) {
	int	fctr = 2;// * @fctr	Conversion factor for KB, for Mb should be fctr = 2048;
	printf("\033[%i;%iH",15,col);
	printf("IOSTATS:");
	printf("\033[%i;%iH",16,col);
	printf("=========");
	int count_printed=0;
	float mticks = sysconf(_SC_CLK_TCK);
	for(int device=0;device<totaldevices;device++){
		if( newdata[device].labelstr[0]!='l'){
			printf("\033[%i;%iH",17+count_printed,col);
			printf(" Device: %s ",newdata[device].labelstr);count_printed++;
			printf("\033[%i;%iH",17+count_printed,col);
			printf("   Transactions/s: %.2f", mticks*((double) (newdata[device].rd_ios + newdata[device].wr_ios) - (double) (olddata[device].rd_ios + olddata[device].wr_ios) )/(itv));
			printf("\033[%i;%iH",17+count_printed+1,col);
			printf("   kb_read/s %7.2f",mticks*(double)(newdata[device].rd_sectors - olddata[device].rd_sectors)/(itv*fctr));
			printf(" kB_wrtn/s %7.2f" ,mticks*(double)(newdata[device].wr_sectors - olddata[device].wr_sectors)/(itv*fctr));
			printf("\033[%i;%iH",17+count_printed+2,col);
			unsigned long long rd_sec = newdata[device].rd_sectors - olddata[device].rd_sectors;
			//if ((newdata[device].rd_sectors < olddata[device].rd_sectors) && (olddata[device].rd_sectors <= -1)) 	rd_sec &= -1;
			unsigned long long wr_sec = newdata[device].wr_sectors - olddata[device].wr_sectors;
			//if ((newdata[device].wr_sectors < olddata[device].wr_sectors) && (olddata[device].wr_sectors <= -1)) 	wr_sec &= -1;
			printf("   kb_read  %8lld", (unsigned long long) rd_sec / fctr);
			printf(" kB_wrtn %8lld\n", (unsigned long long) wr_sec / fctr);
			count_printed+=3;
		}
	}
}



//returns the current time in us
//requires: #include <sys/time.h>
long long int mycurrenttimeb (void) {
	//struct timeval t0;
	//gettimeofday(&t0, NULL);//https://blog.habets.se/2010/09/gettimeofday-should-never-be-used-to-measure-time.html
	//long long int timeus = (long int) (t0.tv_sec *1000000LL + t0.tv_usec );
	struct timespec ts_start;
	clock_gettime(CLOCK_MONOTONIC, &ts_start);
	long long int timeus = (long int) (ts_start.tv_sec*1000000000LL + ts_start.tv_nsec);
	return timeus;
}

// /* read the perf counter from the file descriptor */
// unsigned long long read_perf_counter(int fd) {
// 	unsigned long long single_count[3];
// 	size_t res;
// 	if(fd <= 0)
// 		return 0;
// 	res = read(fd, single_count, 3 * sizeof(unsigned long long));
// 	if(res == 3 * sizeof(unsigned long long))
// 		return single_count[0];
// 	return 0;
// }


unsigned int print_measures_screen(const unsigned int maxcores, struct task_data_t *my_task_data, const unsigned int startline, energy_model param_energy){
	long long int actual_time;
	if (my_task_data->last_end!=0){
		actual_time =my_task_data->last_end;
	}else{
		actual_time=mycurrenttimeb();
	}
	unsigned int i,c;
	char   LIGHT_GREEN[]="\033[1;32m";
	char    LIGHT_BLUE[]="\033[1;34m";
	char    LIGHT_CYAN[]="\033[1;36m";
	char        yellow[]="\033[1;33m";
	char           RED[]="\033[0;31m";
	char     NO_COLOUR[]="\033[0m";
	printf("\033[%i;3H",startline);
	printf("LOAD OF THE TASKs BY PID:\n");
	printf("==========================\n");
	printf(" %sPID",LIGHT_BLUE);
	printf("\t%sTID",LIGHT_GREEN);
	printf("\t%sCORE",RED);
	printf("\t%s%%cpu    %s%%mem",RED,LIGHT_CYAN);
	printf("%s Rchar:  ",yellow);
	printf("Wchar:  ");
	printf("Syscr:  ");
	printf("Syscw:  ");
	printf(" Read:  ");
	printf("Write:  ");
	printf("Cancel wr ");
	printf("Net_send ");
	printf("Net_recv ");
	printf("\n");
// 	really write/read did cause to be fetched from the storage layer
	long long int read_bytes = 0;
	long long int write_bytes = 0;
	long long int cancelled_writes = 0;
	float pid_net_power = 0.0;
	for(i=0;i<my_task_data->totaltid;i++){
		printf("%s%6i",LIGHT_BLUE,my_task_data->subtask[i].pspid);
		printf("  %s%6i",LIGHT_GREEN,my_task_data->subtask[i].pstid);
		if(my_task_data->subtask[i].totaltime==0){
			printf("  %s%6i",RED,my_task_data->subtask[i].currentcore);
			printf("  %s%5.1f",RED,my_task_data->subtask[i].pcpu);
			printf("  %s%5.1f%s",LIGHT_CYAN,my_task_data->subtask[i].pmem,yellow);
			//printf(" %s%9Li us",RED,my_task_data->subtask[i].starttime);
		}else{
			printf("  %s%9lli ms",NO_COLOUR,my_task_data->subtask[i].totaltime/1000000);
			printf("  %5.1f",my_task_data->subtask[i].pmem );
			//printf("  %s%9Li us",NO_COLOUR,my_task_data->subtask[i].finishtime);
			//printf(" %s%9Li us",NO_COLOUR,my_task_data->subtask[i].starttime);
		}
		print_memorysize( my_task_data->subtask[i].rchar);printf(" ");
			print_memorysize( my_task_data->subtask[i].wchar );printf(" ");
		print_memorysize( my_task_data->subtask[i].syscr);printf(" ");
			print_memorysize( my_task_data->subtask[i].syscw );printf(" ");
		print_memorysize( my_task_data->subtask[i].read_bytes );printf(" ");
		print_memorysize( my_task_data->subtask[i].write_bytes);printf(" ");
		print_memorysize( my_task_data->subtask[i].cancelled_write_bytes);printf(" ");

		print_memorysize( my_task_data->subtask[i].send_bytes);printf(" ");
		print_memorysize( my_task_data->subtask[i].rcv_bytes);

			emptyline(1,10);

		//Simple counters of data read and written by the process using functions like read() or write(), including TTY I/O.
		read_bytes += my_task_data->subtask[i].rchar;
		write_bytes += my_task_data->subtask[i].wchar;
		// 	really write/read did cause to be fetched from the storage layer,  to satisfy the process' read request.
		read_bytes += my_task_data->subtask[i].read_bytes;
		write_bytes += my_task_data->subtask[i].write_bytes;
		cancelled_writes += my_task_data->subtask[i].cancelled_write_bytes;


		pid_net_power += (param_energy.E_NET_RCV_PER_MB*my_task_data->subtask[i].rcv_bytes + param_energy.E_NET_SND_PER_MB* my_task_data->subtask[i].send_bytes)* 1.0e-6;
	}
	emptyline(my_task_data->maxtotaltid+1-my_task_data->totaltid,90);


	float total_cpu_energy =0;
	for(c=0;c<maxcores;c++){
		printf("%s TOTAL CORE %2i    load: %s%5.1f %%  %sFreq:%s%8li %sKHz  Energy: %s%6.2f J  %6.2f W",NO_COLOUR,c,
			RED, my_task_data->cores[c].total_load_core, NO_COLOUR,
			RED, my_task_data->cores[c].core_freq, NO_COLOUR,
			RED, my_task_data->cores[c].total_joules_core,
			my_task_data->cores[c].total_watts_core);//,  my_task_data->cores[c].time_between_measures <- in ns
		total_cpu_energy+=my_task_data->cores[c].total_watts_core;
		emptyline(1,15);
	}



	long long int pid_l2_cache_misses=0;
	float pid_mem_power = ((read_bytes + write_bytes - cancelled_writes) / param_energy.L2CACHE_LINE_SIZE + pid_l2_cache_misses) * param_energy.L2CACHE_MISS_LATENCY * param_energy.MEMORY_POWER* 1.0e-9;// / duration;
	/* pid-based disk access power in milliwatt */
// 	float pid_disk_power = (read_bytes * param_energy.E_DISK_R_PER_KB + (write_bytes - cancelled_writes) * param_energy.E_DISK_W_PER_KB);// / (1024 * duration);
	float pid_disk_power = param_energy.hd_power;

	float total_hd_energy=0.0;
	if (my_task_data->first_start!=0) total_hd_energy=pid_disk_power*(actual_time - my_task_data->first_start)/(1.0e9);
	float total_watts= total_cpu_energy + total_hd_energy + pid_mem_power + pid_net_power;
	printf("%s Energy CPU: %s%5.3f W",
		NO_COLOUR, yellow, total_cpu_energy);

	printf("%s + IO: %s%5.3f W",
		NO_COLOUR, yellow, total_hd_energy); //lsat_end and first start are in ns

	printf("%s +MEM: %s%5.3f W",
		NO_COLOUR, yellow, pid_mem_power);

	printf("%s + NET: %s%5.3f W",
		NO_COLOUR, yellow, pid_net_power);

	printf("%s = %s%5.3f W",
		NO_COLOUR, yellow, total_watts);

	printf("%s  ==> (0.25€/KWh)==> %s%2.2f(%5.6f)%s €\n",
		NO_COLOUR, yellow, total_watts*0.25/(1000.0*3600.0),total_watts*0.25/(1000.0*3600.0),
		NO_COLOUR);

	printf("%s TOTAL CPU usage: %s%5.1f %%%s  TOTAL MEM: %s%5.1f %%%s",NO_COLOUR,RED, ((float) my_task_data->total_load_cpu)/ maxcores,NO_COLOUR,LIGHT_CYAN,my_task_data->totalpmem,NO_COLOUR);emptyline(1,20);
	unsigned int linea=startline+4+my_task_data->maxtotaltid+ maxcores +1;
	return linea;
}




void screen_resolution(int *previous_lines,int *previous_cols){
	int curr_lines, curr_cols;
	#ifdef TIOCGSIZE
		struct ttysize ts;
		ioctl(STDIN_FILENO, TIOCGSIZE, &ts);
		curr_cols = ts.ts_cols;
		curr_lines = ts.ts_lines;
	#elif defined(TIOCGWINSZ)
		struct winsize ts;
		ioctl(STDIN_FILENO, TIOCGWINSZ, &ts);
		curr_cols = ts.ws_col;
		curr_lines = ts.ws_row;
	#endif /* TIOCGSIZE */
		if((*previous_lines!=curr_lines) ||(*previous_cols!=curr_cols) )
			cls();
		*previous_lines=curr_lines;
		*previous_cols=curr_cols;
}

//**************************************************************************************************************
//PRINTING IN THE SCREEN
//**************************************************************************************************************


void print_system_stats(struct system_data *mysystem){
	const int col=98+20;
	const int maxrows=16;
	const int start_stats_line=6;
	char NO_COLOUR[]="\033[0m";
	char LIGHT_CYAN[]="\033[1;36m";

	char RED[]="\033[0;31m";
	unsigned int fila=start_stats_line;
	for(int i=0;i<maxrows;i++)
		printf("\033[%i;%iH||",fila+i,col-3);
	printf("\033[%i;%iH",fila++,col);	printf("GLOBAL SYSTEM RESOURCES");	emptyline(1,10);
	printf("\033[%i;%iH",fila++,col);	printf("=======================");	emptyline(1,10);
	printf("\033[%i;%iH",fila++,col);	printf(" Total Memory: %s%i kB%s",LIGHT_CYAN,		mysystem->MemTotal,NO_COLOUR);emptyline(1,5);
	printf("\033[%i;%iH",fila++,col);	printf(" MemFree value is: %s%i kB%s",LIGHT_CYAN,	mysystem->MemFree,NO_COLOUR);emptyline(1,5);
	printf("\033[%i;%iH",fila++,col);	printf(" Buffers value is: %s%i kB%s",LIGHT_CYAN,	mysystem->Buffers,NO_COLOUR);emptyline(1,5);
	printf("\033[%i;%iH",fila++,col);	printf(" Cached value is: %s%i kB%s",LIGHT_CYAN,	mysystem->Cached,NO_COLOUR);emptyline(1,5);
	printf("\033[%i;%iH",fila++,col);	printf(" Used memory: %s%i kB%s", LIGHT_CYAN,		mysystem->usedmemory,NO_COLOUR); emptyline(1,5);
	printf("\033[%i;%iH",fila++,col);	printf(" Used memory: %s%6.2f %%%s", LIGHT_CYAN,	(100.0 * mysystem->usedmemory)/mysystem->MemTotal, NO_COLOUR);
	printf("\033[%i;%iH",fila++,col);	printf(" CPU usage at %s%6.2f %%%s", RED, 			mysystem->cpu_system_load,NO_COLOUR);
	printf("\033[%i;1H",30);
}


unsigned int print_stats(const unsigned int max_painted, const unsigned int active_processes, const int maxcores, int *curr_lines, int *curr_cols,
	char *searchprocess, struct task_data_t *my_task_data_a, struct system_data *mysystem, char *curlcommand, char *to_post, energy_model param_energy){
	//const int col=98;
	//const int maxrows=16;
	const int start_stats_line=6;
	char yellow[]="\033[1;33m", NO_COLOUR[]="\033[0m";
//	char LIGHT_CYAN[]="\033[1;36m";
	char LIGHT_BLUE[]="\033[1;34m";
	char LIGHT_GREEN[]="\033[1;32m";
//	char RED[]="\033[0;31m";
	int i;
	screen_resolution( curr_lines, curr_cols );
	//CLEAN SCREEN
	printf("\033[%i;%iH",1,1);
	int totalcolclean=90+65;
	int totalrowclean=max_painted;
	if (totalcolclean>*curr_cols) totalcolclean=*curr_cols-1;
	if (totalrowclean>*curr_lines) totalrowclean=*curr_lines-1;
	emptyline(totalrowclean,totalcolclean);
	//**************************************************************************************************************
	//CABECERA
	printf("\033[%i;1H",1);
	printf(" %s(C) 2017 J.M. Montanana, High Performance Computing Center (HLRS), Stuttgart%s",LIGHT_GREEN,NO_COLOUR);
	printf("\033[%i;1H",2);
	printf(" %s  Developed under the financing of the European Research Council in the EU-project PHANTOM%s",LIGHT_GREEN,NO_COLOUR);
	printf("\033[%i;3H",3);
	printf("MONITORING THE TASK: %s%s%s",LIGHT_BLUE,searchprocess,NO_COLOUR);
	for(i=longitud(searchprocess);i<*curr_cols ;i++) printf(" ");
	printf("\033[%i;3H",4);
	if((my_task_data_a->first_start!=0)&&(my_task_data_a->last_end!=0)){
		printf(" TOTAL EXECUTION TIME:%9lli s ", (my_task_data_a->last_end - my_task_data_a->first_start)/1000000000LL);
			long long int temp_time = (my_task_data_a->last_end - my_task_data_a->first_start)%1000000000LL;
		printf(" +  %3lli ms ", temp_time/1000000);
			temp_time = (my_task_data_a->last_end - my_task_data_a->first_start)%1000000LL;
		printf(" +  %3lli us ", temp_time/1000);
			temp_time = (my_task_data_a->last_end - my_task_data_a->first_start)%1000LL;
		printf(" +  %3lli ns ", temp_time);
	}
	unsigned int fila=start_stats_line+9;
	unsigned int linea= (active_processes> 0) ? 2+ active_processes +7 : 2; //length of the curl request
	linea+=start_stats_line+4+my_task_data_a->maxtotaltid+ maxcores +1;
	unsigned int local_max_painted = (linea> fila) ? linea : fila;
	printf("\033[%i;3H",local_max_painted);
	printf("%s Press any key to exit%s",yellow,NO_COLOUR);//emptyline(1,10);
	//**************************************************************************************************************
	//TASK'S STATS
	//**************************************************************************************************************
	linea=print_measures_screen( maxcores, my_task_data_a, start_stats_line, param_energy);
	//if(argc>3) linea=print_measures_screen( maxcores, &my_task_data_b, linea+1);
	//print_disk_stats( newdata, olddata, totaldevices, col,itv);
	//fila=start_stats_line;
	//for(int i=0;i<maxrows;i++)
	//	printf("\033[%i;%iH||",fila+i,col-3);
	//printf("\033[%i;%iH",fila++,col);	printf("GLOBAL SYSTEM RESOURCES");	emptyline(1,10);
	//printf("\033[%i;%iH",fila++,col);	printf("=======================");	emptyline(1,10);
	//printf("\033[%i;%iH",fila++,col);	printf(" Total Memory: %s%i kB%s",LIGHT_CYAN,		mysystem->MemTotal,NO_COLOUR);emptyline(1,5);
	//printf("\033[%i;%iH",fila++,col);	printf(" MemFree value is: %s%i kB%s",LIGHT_CYAN,	mysystem->MemFree,NO_COLOUR);emptyline(1,5);
	//printf("\033[%i;%iH",fila++,col);	printf(" Buffers value is: %s%i kB%s",LIGHT_CYAN,	mysystem->Buffers,NO_COLOUR);emptyline(1,5);
	//printf("\033[%i;%iH",fila++,col);	printf(" Cached value is: %s%i kB%s",LIGHT_CYAN,	mysystem->Cached,NO_COLOUR);emptyline(1,5);
	//printf("\033[%i;%iH",fila++,col);	printf(" Used memory: %s%i kB%s", LIGHT_CYAN,		mysystem->usedmemory,NO_COLOUR); emptyline(1,5);
	//printf("\033[%i;%iH",fila++,col);	printf(" Used memory: %s%6.2f %%%s", LIGHT_CYAN,	(100.0 * mysystem->usedmemory)/mysystem->MemTotal, NO_COLOUR);
	//printf("\033[%i;%iH",fila++,col);	printf(" CPU usage at %s%6.2f %%%s", RED, 			mysystem->cpu_system_load,NO_COLOUR);
	//**************************************************************************************************************
	//CURL request
	print_system_stats(mysystem);
	printf("\033[%i;1H",linea+2); 		printf("%s%s %s\n",LIGHT_BLUE,						curlcommand,NO_COLOUR);
	printf("\033[%i;1H",linea+3); 		printf("%s%s %s\n",LIGHT_BLUE,						to_post,NO_COLOUR);
	return local_max_painted;
}
